/*
 *  Avr_Interfaz_USB.c
 *  
 *	Autor: Rubn Flores A.
 *
 *	Proyecto: Diseo de una interfaz para sistemas de control en Java
 *	con comunicacin inalmbrica utilizando un microcontrolador AVR
 *	con interfaz USB.
 *
 *	Microcontrolador: AVR ATMEGA 8
 *	Fusibles uilizados: 
 *	
 *	Low Byte:	0xFF
 *	High Byte:	0xC9
 *
 */ 

/* Frecuencia del cristal: 12 MHz */
// Nota: Los valores permitidos para V-USB pueden ser: 12 MHz, 12.8 MHz, 15 MHz, 16 MHz,  16.5 MHz, 18 MHz y 20 MHz. 

#define F_CPU 12000000UL

/* Libreras genricas utilizadas  */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/sfr_defs.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdbool.h>
#include <stdlib.h>
#include <math.h>

/* Libreras especiales utilizadas  */

// Librera creada, contiene definiciones y funciones utilizadas para configurar el Transceiver NRF24L01. 
// Nota: modificar libreria si se desea modificar los pines utilizados para otro tipo de AVR.
#include "NRF24L01.h" 

/* Libreras utilizadas por VUSB.*/
#include "usbdrv.h"
#include "oddebug.h"
#include "usbconfig.h"
#include "usbportability.h"

/* Definiciones de constantes utilizadas. */

// constantes que identifican el comando recibido en bRequest.
#define usbReq_Send_RF_Data					1 
#define usbReq_Receive_USB_Data				2
#define usbReq_Receive_RF_Data				3 //Obsoleto. Utilizar usbReq_Read_RF_Data.
#define usbReq_Send_Dynamic_RF_Data			4
#define usbReq_Read_Dynamic_RF_Data			5
#define usbReq_Read_RF_Data					8
#define usbReq_Enable_DPL					10
#define usbReq_Disable_DPL					11
#define usbReq_Set_TX_Mode					12
#define usbReq_Set_TX_Address				13
#define usbReq_Get_TX_Address				14
#define usbReq_Set_RX_Mode					15
#define usbReq_Set_RX_Address				16
#define usbReq_Get_RX_Address				17
#define usbReq_Set_TX_Length				18
#define usbReq_Set_RX_Length				19
#define usbReq_Get_TX_Length				20
#define usbReq_Get_RX_Length				21
#define usbReq_Get_DPL_State				22
#define usbReq_Get_RF_Mode					23
#define usbReq_Read_nRF24L01_Register		30
#define usbReq_Write_nRF24L01_Register		31
#define usbReq_Set_RF_Channel				50
#define usbReq_Get_RF_Channel				51
#define usbReq_Set_Data_Speed				60
#define usbReq_Get_Data_Speed				61
#define usbReq_Turn_LED_On					200
#define usbReq_Turn_LED_Off					201
#define usbReq_Blink_LED					202
#define usbReq_Set_Initial_TX_Address		210
#define usbReq_Set_Initial_RX_Address		215
#define usbReq_Set_Initial_RF_Channel		220
#define usbReq_Set_Initial_RF_Speed			225
#define usbReq_Set_Initial_RF_Mode			230
#define usbReq_Set_Initial_TX_Length		235
#define usbReq_Set_Initial_RX_Length		240
#define usbReq_Set_Initial_DPL_State		245
#define usbReq_Get_Initial_Values			250

// constantes para identificacin de errores.
#define Error				64
#define Ok					20
#define sensores			6

#define LED_IND				PC5
#define LED_ON				PORTC |= 0X20
#define LED_OFF				PORTC &= 0XDF

/* Definiciones de variables globales utilizadas. */
static unsigned char USB_Buffer[32];
static unsigned char RF_Data_Buffer[32];
static unsigned char ACK_Data_Buffer[32];
static unsigned char Initial_Values_Buffer[16];
static unsigned char RF_Channel[1];
static unsigned char RF_Speed[1];
static unsigned char Read_Register[1];
static unsigned char TX_AD[5];
uint8_t TX_Length;
static unsigned char RX_AD[5];
uint8_t RX_Length;

// Utilizados para Leer Datos del USB
static unsigned char dataReceived = 0, dataLength = 0;

unsigned char Status_temp;
bool DPL_State = false;
// TX = true; RX = false
bool RF_Mode = false;
#define TX_Mode	true
#define RX_Mode	false

uint8_t ACK_Data_Width;
unsigned char i;
uint8_t reg_data[1];

/* Variables de inicio almacenadas en la EEPROM*/
uint8_t			EEMEM	NonVolatileRFMode = 1;
uint8_t			EEMEM	NonVolatileRFChannel = 60;
unsigned char	EEMEM	NonVolatileRFSpeed = 1;
unsigned char	EEMEM	NonVolatileTXAddress [5] = "COM01";
uint8_t			EEMEM	NonVolatileTXDataLength = 4;
unsigned char	EEMEM	NonVolatileRXAddress [5] = "USBRF";
uint8_t			EEMEM	NonVolatileRXDataLength = 32;
uint8_t			EEMEM	NonVolatileDPLState = 1;



uint8_t bRequest_Send_Dynamic_RF_Data(uint8_t leng);
uint8_t bRequest_Read_Dynamic_RF_Data();
bool bRequest_Send_Data_RF();
bool bRequest_Read_RF_Data();



/*	Funcin usbFunctionSetup.
	Esta funcin es llamada automticamente cuando existe un 
	mensaje de control  por parte del host.
*/
USB_PUBLIC unsigned char usbFunctionSetup(unsigned char data[8])
{

	// Obtiene los 8 Bytes del paquete Setup
	// y los almacena en el apuntador rq.
	usbRequest_t *rq = (void *)data; 
	

	
	/*  El valor del comando se encuentra en el campo 'bRequest', por lo que
		se evala su valor y de acuerdo a ste se realiza la funcin deseada. 		
	*/
	
	switch(rq->bRequest) 
	{ 
		case usbReq_Send_RF_Data:
		if (bRequest_Send_Data_RF() == true)
		{
			usbMsgPtr = "1";
			return 1;
		}
		
		else
		return 0;
		
		
		case usbReq_Receive_USB_Data:
		bRequest_Receive_Data_USB((unsigned char)rq->wLength.word);
		return USB_NO_MSG;		// Llama la funcin usbFunctionWrite.
		
		case usbReq_Receive_RF_Data:
		bRequest_Receive_Data_RF();
		usbMsgPtr = RF_Data_Buffer;
		return sizeof(RF_Data_Buffer);
		
		case usbReq_Send_Dynamic_RF_Data:
		ACK_Data_Width = bRequest_Send_Dynamic_RF_Data((uint8_t) rq->wValue.word);

		if (ACK_Data_Width== Error || ACK_Data_Width >32)
		{
			ACK_Data_Buffer[0] = 'E';
			ACK_Data_Buffer[1] = 'R';
			ACK_Data_Buffer[2] = 'R';
			ACK_Data_Buffer[3] = '0';
			ACK_Data_Buffer[4] = '0';
			usbMsgPtr = ACK_Data_Buffer;
			return 5;
		}
		
		
		
 		usbMsgPtr = ACK_Data_Buffer;
 		return ACK_Data_Width;
		
	
		case usbReq_Set_RF_Channel:
		bRequest_Set_Channel_RF((uint8_t) rq->wValue.word);
		usbMsgPtr = RF_Channel;
		return sizeof(RF_Channel);
		
		case usbReq_Get_RF_Channel:
		bRequest_Get_Channel_RF();
		usbMsgPtr = RF_Channel;
		return sizeof(RF_Channel);
		
		case usbReq_Set_Data_Speed:
		bRequest_Set_Data_Speed(rq->wValue.word);
		usbMsgPtr = RF_Speed;
		return sizeof(RF_Speed);
		
		case usbReq_Get_Data_Speed:
		bRequest_Get_RF_Speed();
		usbMsgPtr = RF_Speed;
		return sizeof(RF_Speed);
		
		case usbReq_Enable_DPL:
		bRequest_Enable_DPL();
		return 0;
		
		case usbReq_Disable_DPL:
		bRequest_Disable_DPL();
		return 0;
		
		case usbReq_Set_TX_Mode:
		RF_Mode = Set_TX_Mode();
		return 0;
		
		case usbReq_Set_RX_Mode:
		RF_Mode = Set_RX_Mode();
		return 0;
		
		case usbReq_Set_TX_Address:
		bRequest_Set_TX_Address();
		return 0;
		
		case usbReq_Set_Initial_TX_Address:
		bRequest_Set_Initial_TX_Address();
		return 0;
		
		case usbReq_Set_Initial_RF_Channel:
		bRequest_Set_Initial_RF_Channel(rq->wValue.word);
		return 0;
		
		case usbReq_Set_Initial_RF_Speed:
		bRequest_Set_Initial_RF_Speed(rq->wValue.word);
		return 0;
		
		case usbReq_Set_Initial_RX_Length:
		bRequest_Set_Initial_RX_Length(rq->wValue.word);
		return 0;
		
		case usbReq_Set_Initial_TX_Length:
		bRequest_Set_Initial_TX_Length(rq->wValue.word);
		return 0;
		
		case usbReq_Set_Initial_RF_Mode:
		bRequest_Set_Initial_RF_Mode(rq->wValue.word);
		return 0;
		
		case usbReq_Set_Initial_DPL_State:
		bRequest_Set_Initial_DPL_State(rq->wValue.word);
		return 0;
		
		case usbReq_Get_TX_Address:
		bRequest_Get_TX_Address();
		usbMsgPtr = TX_AD;
		return sizeof(TX_AD);
		
		case usbReq_Set_RX_Address:
		bRequest_Set_RX_Address();
		return 0;
		
		case usbReq_Set_Initial_RX_Address:
		bRequest_Set_Initial_RX_Address();
		return 0;
		
		case usbReq_Get_RX_Address:
		bRequest_Get_RX_Address();
		usbMsgPtr = RX_AD;
		return sizeof(RX_AD);
		
		case usbReq_Set_TX_Length:
		bRequest_Set_TX_Length(rq->wValue.word);
		usbMsgPtr = TX_Length;
		return 1;
		
		case usbReq_Get_TX_Length:
		usbMsgPtr = TX_Length;
		return 1;
		
		case usbReq_Set_RX_Length:
		bRequest_Set_RX_Length(rq->wValue.word);
		usbMsgPtr = RX_Length;
		return 1;
		
		case usbReq_Get_RX_Length:
		usbMsgPtr = RX_Length;
		return 1;
		
		case usbReq_Get_DPL_State:
		if (DPL_State == false)
		{
			return 0;	
		}
		else
		usbMsgPtr = "1";
		return 1;
		
		case usbReq_Get_RF_Mode:
		if (RF_Mode == false)
		{
			return 0;
		}
		else
		usbMsgPtr = "1";
		return 1;
		
		case usbReq_Write_nRF24L01_Register:
		bRequest_Write_Register(rq->wValue.bytes[0], rq->wIndex.bytes[0]);
		return 0;
		
		case usbReq_Read_nRF24L01_Register:
		bRequest_Read_Register(rq->wValue.word, &reg_data);
		usbMsgPtr = reg_data;
		return sizeof(reg_data);
		
		case usbReq_Turn_LED_On:
		LED_ON;
		return 0;
		
		case usbReq_Turn_LED_Off:
		LED_OFF;
		return 0;
		
		case usbReq_Blink_LED:
		Blink_LED();
		return 0;
		
		case usbReq_Get_Initial_Values:
		bRequest_Get_Initial_Values(&Initial_Values_Buffer);
		usbMsgPtr = Initial_Values_Buffer;
		return sizeof(Initial_Values_Buffer);
		
		case usbReq_Read_RF_Data:
		if (bRequest_Read_RF_Data() == false)
		{
			return 0;
		}
		usbMsgPtr = RF_Data_Buffer;
		return RX_Length;
		
		case usbReq_Read_Dynamic_RF_Data:
		ACK_Data_Width = bRequest_Read_Dynamic_RF_Data();
		if (rq->wValue.word > 0)
		{
		Set_ACK_Data(USB_Buffer);	
		}
		
		if (ACK_Data_Width == 0)
		{
			return 0;
		}
		usbMsgPtr = RF_Data_Buffer;
		return ACK_Data_Width;
		
		
		default:
		return 0;
	}
	
	return 0;
}


/*	Funcin usbFunctionWrite
	Esta funcin es llamada cuando se reciben datos del host (PC) hacia el dispositivo.
*/

USB_PUBLIC unsigned char usbFunctionWrite(unsigned char *data, unsigned char len) {
	unsigned char i;
	
	for(i = 0; dataReceived < dataLength && i < len; i++, dataReceived++)
	USB_Buffer[dataReceived] = data[i];
	// regresa 1 s recibimos todo, 0 s no.
	return (dataReceived == dataLength); 
	
	
}


/* Funcin main*/

int main(void)
{
	// Variable local, utilizada para la re-enumeracin.
	unsigned char i;
	
	/* Configuracin de los puertos */

	DDRB |= (1<<CE) | (1<<CSN) | (1<<MOSI) | (1<<SCK); 
	DDRB &= ~(1<<MISO);
	DDRD &= ~(1<<IRQ);
	DDRC |= (1<<LED_IND);

	
	// Se asegura que no est activado el pin Slave del nRF24L01.
	RF_Select_Not;
	
	// Habilita el Watchdog Timer con 1 segundo.
	wdt_enable(WDTO_1S); 

	usbInit();
	
	// Fuerza la re-enumeracin.
	usbDeviceDisconnect(); 
	for(i = 0; i<250; i++)
	{
		// Espera 500ms.
		wdt_reset(); 
		_delay_ms(2);
	}
	usbDeviceConnect();
	
	// Inicializa la interfaz SPI
	
	SPI_Init();
	
	/* Obtiene los valores iniciales de la EEPROM*/
	
	Get_Initial_Values();
	
    /* Inicializa el mdulo NRF24L01+
	*  En caso de no tener algun valor definido, 
	*  los valores por Default son los siguientes:
	*  Modo: Transmisor
	*  Direccin de TX: COM01
	*  Direccin de RX: USBRF
	*  Canal de RF: 60 (2460 MHz)
    */
	
	NRF24L01_Init(RF_Mode, TX_AD, RX_AD, RF_Channel[0], TX_Length, RX_Length, RF_Speed[0]);
	
	// Habilita el modo "bloques de datos dinmicos" (DPL) 
	if (DPL_State == true)
	{
		DPL_State = Enable_DPL();
	}

	
	// Parpadea LED como indicador de que todo est configurado
	
	Blink_LED();
	
	// Habilitacin general de interrupciones
	
	sei();                
	
	// Bucle infinito, a la espera de peticiones USB.
	
	while (1)
	{
		wdt_reset(); 
		usbPoll();	
	}
}

void Get_Initial_Values()
{
	uint8_t RFMode;
	RFMode = eeprom_read_byte (& NonVolatileRFMode);
	if (RFMode == 0xFF)
	{
		RF_Channel[0] = 60;
		RF_Mode = true;
		TX_Length =4;
		RX_Length =32;
		DPL_State = true;
		RF_Speed[0] = 1;
		
		TX_AD[0] = "C";
		TX_AD[1] = "O";
		TX_AD[2] = "M";
		TX_AD[3] = "0";
		TX_AD[4] = "1"; 
		
		RX_AD[0] = "U";
		RX_AD[1] = "S";
		RX_AD[2] = "B";
		RX_AD[3] = "R";
		RX_AD[4] = "F";
		return;
	}
	else if (RFMode == 0x00)
	{
		RF_Mode = false;
	}
	else if (RFMode == 0x01)
	{
		RF_Mode = true;
	}
	RF_Channel[0] = eeprom_read_byte(& NonVolatileRFChannel);
	RF_Speed[0] = eeprom_read_byte(& NonVolatileRFSpeed);
	TX_Length = eeprom_read_byte(& NonVolatileTXDataLength);
	RX_Length = eeprom_read_byte(& NonVolatileRXDataLength);
	eeprom_read_block (( void *) TX_AD , ( const void *) NonVolatileTXAddress , sizeof(TX_AD));
	eeprom_read_block (( void *) RX_AD , ( const void *) NonVolatileRXAddress , sizeof(RX_AD));
	uint8_t DPL;
	DPL = eeprom_read_byte (& NonVolatileDPLState);
	if (DPL == 0)
	{
		DPL_State = false;
	}
	else
	DPL_State = true;
}

bool bRequest_Send_Data_RF()
{
	if (RF_Mode==RX_Mode)
	{
		RF_Mode = Set_TX_Mode();
		_delay_ms(2);
	}

	
	Transmit_RF_Data(USB_Buffer);				
	while(bit_is_set(PIND,IRQ_PIN));
	Status_temp=Get_Status();
	if(bit_is_set(Status_temp,MAX_RT))
	{
		Clean_Status();
		return false;
	}
	Clean_Status();
	return true;
}

void bRequest_Receive_Data_USB(unsigned char leng)
{
	dataLength  = leng;
	dataReceived = 0;
	if(dataLength  > sizeof(USB_Buffer)) 
	dataLength  = sizeof(USB_Buffer);
}

void bRequest_Receive_Data_RF()
{
	uint16_t j;
	Transmit_RF_Data(USB_Buffer);
	while(bit_is_set(PIND,IRQ_PIN));
	Status_temp=Get_Status();
	Clean_Status();
	RF_Data_Buffer[16]=Error;
	
	
	if(bit_is_set(Status_temp,MAX_RT))
	{
		RF_Data_Buffer[16]=Error;
		return 0;
	}
	Set_RX_Mode();
	
	_delay_ms(2);

	for (j=0;j<=60000;j++)
	{
		if((bit_is_clear(PIND,IRQ_PIN)))
		{
			RF_CE_Low;
			RF_Data_Buffer[16]=Ok;
			Read_RF_Data(&RF_Data_Buffer);
			Clean_Status;
			j=61000;
		}
	}
	Set_TX_Mode();
	_delay_ms(2);
}

uint8_t bRequest_Send_Dynamic_RF_Data(uint8_t leng)
{
	
	if ((DPL_State == true) && (RF_Mode == true))
	{
		uint16_t j;
		ACK_Data_Width = 0;
		Transmit_Dynamic_RF_Data(USB_Buffer,leng);
		while(bit_is_set(PIND,IRQ_PIN));
		Status_temp=Get_Status();
		
		
		if((bit_is_set(Status_temp,MAX_RT)))
		{
			Clean_Tx_Fifo();
			Clean_Status();
			return Error;
		}
		
		if((bit_is_set(Status_temp,RX_DR)))
		{
			ACK_Data_Width = Read_Dynamic_RF_Data(&ACK_Data_Buffer);
			Clean_Rx_Fifo();
		}
		
		Clean_Status();	
 		return ACK_Data_Width;
	}
		
	return 0;
}

void bRequest_Set_Channel_RF(uint8_t Can_RF)
{
	if (Can_RF >= 0 && Can_RF <=127)
	{
		Set_RF_Channel(Can_RF);
	}
	
	RF_Channel[0] = Get_RF_Channel();
}

void bRequest_Get_Channel_RF()
{
	RF_Channel[0] = Get_RF_Channel();
}

void bRequest_Set_Data_Speed(uint8_t Velocidad_RF)
{
	if (Velocidad_RF==1)
	{
		RF_Speed[0]=RF_SETUP_SPEED_2Mbps;
	}
	else if (Velocidad_RF==2)
	{
		RF_Speed[0]=RF_SETUP_SPEED_1Mbps;
	}
	else if (Velocidad_RF==3)
	{
		RF_Speed[0]=RF_SETUP_SPEED_250Kbps;
	}
	else
	{
		return;
	}
	
	Set_RF_Dataspeed(RF_Speed[0]);
}

void bRequest_Get_RF_Speed()
{
	RF_Speed[0] = Get_RF_Dataspeed();
}

void Blink_LED()
{
	PORTC |= 0x20;
	_delay_ms(50);
	PORTC &= 0xDF;
}

void bRequest_Enable_DPL()
{
	DPL_State = Enable_DPL();
}

void bRequest_Disable_DPL()
{
	DPL_State = Disable_DPL();
}

void bRequest_Set_TX_Address()
{
for(i=0;i<5;i++)
{
	TX_AD[i] = USB_Buffer[i];
}
Set_TX_Address(TX_AD);
}

void bRequest_Get_TX_Address()
{
	Get_TX_Address(&TX_AD);
}

void bRequest_Set_RX_Address()
{
	for(i=0;i<5;i++)
	{
		RX_AD[i] = USB_Buffer[i];
	}
	Set_RX_Address(RX_AD);
}

void bRequest_Get_RX_Address()
{
	Get_RX_Address(&RX_AD);
}

void bRequest_Write_Register(uint8_t reg, uint8_t register_data)
{
	RF_Select;
	SPI_Send(W_REGISTER | reg);
	SPI_Send(register_data);
	RF_Select_Not;
}

void bRequest_Read_Register(uint8_t reg, uint8_t register_data[])
{
	uint8_t *rgdata = register_data;
	RF_Select;
	SPI_Send(R_REGISTER|reg);
	rgdata[0] = SPI_Send(NOP_RF);
	RF_Select_Not;
}

void bRequest_Set_Initial_TX_Address()
{
	unsigned char temp_TX_Add[5];
	for(i=0;i<5;i++)
	{
		temp_TX_Add[i] = USB_Buffer[i];
	}
	eeprom_update_block (( const void *) temp_TX_Add , ( void *) NonVolatileTXAddress, 5);
}

void bRequest_Set_Initial_RX_Address()
{
	unsigned char temp_RX_Add[5];
	for(i=0;i<5;i++)
	{
		temp_RX_Add[i] = USB_Buffer[i];
	}
	eeprom_update_block (( const void *) temp_RX_Add , ( void *) NonVolatileRXAddress, 5);
}

void bRequest_Set_Initial_RF_Channel(uint8_t Channel)
{
	eeprom_update_byte( &NonVolatileRFChannel, Channel);
}

void bRequest_Set_Initial_RF_Speed(unsigned char Speed)
{
	eeprom_update_byte( &NonVolatileRFSpeed, Speed);
}

void bRequest_Set_Initial_RX_Length(uint8_t AD_P1_Length)
{
	eeprom_update_byte( &NonVolatileRXDataLength, AD_P1_Length);
}

void bRequest_Set_Initial_TX_Length(uint8_t AD_P1_Length)
{
	eeprom_update_byte( &NonVolatileTXDataLength, AD_P1_Length);
}

void bRequest_Set_Initial_RF_Mode(uint8_t Mode)
{
	eeprom_update_byte( &NonVolatileRFMode, Mode);
}

void bRequest_Set_Initial_DPL_State(uint8_t DPL)
{
	eeprom_update_byte( &NonVolatileDPLState, DPL);
}

void bRequest_Set_TX_Length(uint8_t AD_P0)
{
	Set_TX_Length(AD_P0);
	TX_Length = AD_P0;
}

void bRequest_Set_RX_Length(uint8_t AD_P1)
{
	Set_TX_Length(AD_P1);
	RX_Length = AD_P1;
}

void bRequest_Get_Initial_Values(unsigned char Values[16])
{
	unsigned char *val = Values;
	unsigned char Temp_AD[5];
		
	val[0] = eeprom_read_byte(&NonVolatileRFMode);
	val[1] = eeprom_read_byte(&NonVolatileRFChannel);
	val[2] = eeprom_read_byte(&NonVolatileRFSpeed);
	eeprom_read_block((void*)Temp_AD, (const void*)NonVolatileTXAddress,5);
	
	for (i=0;i<5;i++)
	{
		val[i+3]= Temp_AD[i];
	}
	
	val[8] = eeprom_read_byte(&NonVolatileTXDataLength);
	
	eeprom_read_block((void*)Temp_AD, (const void*)NonVolatileRXAddress,5);
	for (i=0;i<5;i++)
	{
		val[i+9]= Temp_AD[i];
	}
	
	val[14] = eeprom_read_byte(&NonVolatileRXDataLength);
	val[15] = eeprom_read_byte(&NonVolatileDPLState);
	
}

bool bRequest_Read_RF_Data()
{
	if (RF_Mode == TX_Mode)
	{
		RF_Mode = Set_RX_Mode();
		_delay_ms(2);
	}
	
	Status_temp=Get_Status();

	if(bit_is_set(Status_temp,RX_DR))
		{
			RF_CE_Low;
			Read_RF_Data(&RF_Data_Buffer);
			Clean_Status();
			Clean_Rx_Fifo();
			return true;
		}
	
	return false;
}

uint8_t bRequest_Read_Dynamic_RF_Data()
{
	if (RF_Mode == TX_Mode)
	{
		RF_Mode = Set_RX_Mode();
		_delay_ms(2);
	}
	
	Status_temp=Get_Status();

	if(bit_is_set(Status_temp,RX_DR))
	{
		RF_CE_Low;
		uint8_t qty;
		qty = Read_Dynamic_RF_Data(&RF_Data_Buffer);
		Clean_Status();
		Clean_Rx_Fifo();
		RF_CE_High;
		return qty;
	}
	
	return 0;
}




