/*
 *  Nombre: NRF24L01.c
 *  
 *	Autor: Rubn Flores A.
 *
 *	Proyecto: Diseo de una interfaz para sistemas de control en Java
 *	con comunicacin inalmbrica utilizando un microcontrolador AVR
 *	con interfaz USB.
 *
 *	Descripcin: Archivo fuente que contiene las funciones de
 *	control y configuracin del NRF24L01+, requiere del archivo
 *  de cabecera NRF24L01.h
 *	
 */ 

/* Definiciones de reloj y librerias utilizadas*/
#define F_CPU 12000000UL

#include "NRF24L01.h"
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdbool.h>

/*	Variables que alojan el valor de longitud de los bloques de datos de 
	recepcin y transmisin (para bloques de datos estticos) */

uint8_t PP0_Length, PP1_Length;

/* Inicializa la interfaz SPI */
void SPI_Init()
{
	SPCR |= (1<<SPE)|(1<<MSTR);
}

/* Enva un dato utilizando el SPI */
unsigned char SPI_Send(unsigned char tData)
{
	unsigned char datain;	
	/* Envia Transmisin SPI */
	SPDR = tData;
	/* Espera a que termine de enviar la transmision */
	while(!(SPSR & (1<<SPIF)));
	/* Obtiene valor del pin MISO. */
	datain = SPDR;
	/* Regresa el valor adquirido. */
	return datain;
}

/* Regresa el valor del registro STATUS del nRF24L01+ */
unsigned char Get_Status()
{
	unsigned char Status_temp;
	RF_Select;
	SPI_Send(R_REGISTER|STATUS_RF);
	RF_Select_Not;
		
	RF_Select;
	Status_temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	return Status_temp;
}

/* Limpia el FIFO de TX del nRF24L01+ */
void Clean_Tx_Fifo()
{
	RF_Select;
	SPI_Send(FLUSH_TX); 
	RF_Select_Not;
}

/* Limpia el valor del registro STATUS del nRF24L01+ */
void Clean_Status()
{
	unsigned char Status_temp;
	Status_temp=Get_Status();
	Status_temp &= 0x7F;
	RF_Select;
	SPI_Send(W_REGISTER|STATUS_RF);
	SPI_Send(Status_temp);
	RF_Select_Not;
}


/* Regresa el valor del registro FIFO_STATUS del nRF24L01+ */
unsigned char Get_Fifo_Status()
{
	unsigned char fifo_temp;
	RF_Select;
	SPI_Send(R_REGISTER|FIFO_STATUS);
	fifo_temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	return fifo_temp;
}

/* Limpia el FIFO de RX del nRF24L01+ */
void Clean_Rx_Fifo()
{
	RF_Select;
	SPI_Send(FLUSH_RX); //Limpia FIFO RX.
	RF_Select_Not;
}

/* Entra al modo apagado del nRF24L01+ */
void Set_Power_Down_Mode()
{
	unsigned char Config_Temp;
	
	RF_Select;
	SPI_Send(R_REGISTER|CONFIG_RF); 
	Config_Temp = SPI_Send(0xFF); 
	RF_Select_Not;
	
	Config_Temp &= ~(1<<1); 
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(Config_Temp); 
	RF_Select_Not;
}

/* Entra al modo encendido del nRF24L01+ */
void Set_Power_Up_Mode()
{
	unsigned char Config_Temp;
	
	RF_Select;
	SPI_Send(R_REGISTER|CONFIG_RF);
	Config_Temp = SPI_Send(0x00);
	RF_Select_Not;
	
	Config_Temp |= (1<<1); 
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(Config_Temp); 
	RF_Select_Not;
}

/* Entra al modo RX del nRF24L01+ */
bool Set_RX_Mode()
{
	RF_CE_Low;
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(0x3F);
	RF_Select_Not;
	
	RF_CE_High;
	
	_delay_us(130);
	
	return false;
}

/* Entra al modo TX del nRF24L01+ */
bool Set_TX_Mode()
{
	RF_CE_Low;
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(0x4E); 
	RF_Select_Not;
	return true;
}

/* Configura el canal RF (valores permitidos de 0 - 127)*/
void Set_RF_Channel(uint8_t RF_CHANNEL)
{
	if(RF_CHANNEL>127 || RF_CHANNEL< 0)
	{
		return;
	}
	
	RF_Select;
	SPI_Send(W_REGISTER|RF_CH); 
	SPI_Send(RF_CHANNEL);
	RF_Select_Not;
}

/* Regresa el canal RF*/
uint8_t Get_RF_Channel()
{
	uint8_t canal;
	RF_Select;
	SPI_Send(R_REGISTER|RF_CH); 
	canal= SPI_Send(NOP_RF);
	RF_Select_Not;
	
	return canal;
}

/* Configura la velocidad de tranmisin RF 
	Valores permitidos (definidos en nRF24L01.h) : 
	RF_SETUP_SPEED_1Mbps	
	RF_SETUP_SPEED_2Mbps	
	RF_SETUP_SPEED_250Kbps	
*/
void Set_RF_Dataspeed(uint8_t velocidad)
{
	RF_Select;
	SPI_Send(W_REGISTER|RF_SETUP); 
	SPI_Send(velocidad);
	RF_Select_Not;
}

/* Regresa la velocidad de tranmisin RF 
	Valores de regreso: 	
	2Mbps	=	1
	1Mbps	=	2
	250Kbps =	3
*/
uint8_t Get_RF_Dataspeed()
{
	uint8_t velocidad;
	RF_Select;
	SPI_Send(R_REGISTER|RF_SETUP); 
	velocidad = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	velocidad &= 0xFE; 
	
 	if (velocidad==RF_SETUP_SPEED_2Mbps)
 	{
 		velocidad=1;
 	}
 	else if (velocidad==RF_SETUP_SPEED_1Mbps)
 	{
 		velocidad=2;
 	}
 	else if (velocidad==RF_SETUP_SPEED_250Kbps)
 	{
 		velocidad=3;
 	}
	
	return velocidad;
	
}


/* Configura la direccin de TX (5 Bytes) */
void Set_TX_Address(unsigned char ADDR_P0[5])
{
	uint8_t i;
	
	RF_Select;
	SPI_Send(W_REGISTER|TX_ADDR); 
	for (i=0;i<5;i++)
	{
		SPI_Send(ADDR_P0[i]);
	}
	RF_Select_Not;
	
	RF_Select;
	SPI_Send(W_REGISTER|RX_ADDR_P0); 
	for (i=0;i<5;i++)
	{
		SPI_Send(ADDR_P0[i]);
	}
	RF_Select_Not;
}

void Get_TX_Address(unsigned char ADDR_P0[5])
{
	uint8_t i;
	unsigned char *ptr = ADDR_P0;
	RF_Select;
	SPI_Send(R_REGISTER|TX_ADDR);
	for (i=0;i <5; i++)
	{
		ptr[i] = SPI_Send(NOP_RF);
	}		
	RF_Select_Not;
}

void Get_RX_Address(unsigned char ADDR_P1[5])
{
	uint8_t i;
	unsigned char *ptr = ADDR_P1;
	RF_Select;
	SPI_Send(R_REGISTER|RX_ADDR_P1);
	for (i=0;i <5; i++)
	{
		ptr[i] = SPI_Send(NOP_RF);
	}
	RF_Select_Not;
}


/* Configura la longitud de los bloques de datos de TX (0-32 Bytes) */
void Set_TX_Length(uint8_t P0_LENGTH)
{
	RF_Select;
	SPI_Send(W_REGISTER|RX_PW_P0); //Envia Comando de configuracion de Pipe 0.
	SPI_Send(P0_LENGTH); //Pipe 0.
	RF_Select_Not;
	
	PP0_Length = P0_LENGTH;
}

/* Configura la direccin de RX (5 Bytes) */
void Set_RX_Address(unsigned char ADDR_P1[5])
{
	uint8_t i;
	RF_Select;
	SPI_Send(W_REGISTER|RX_ADDR_P1); 
	for (i=0;i<5;i++)
	{
		SPI_Send(ADDR_P1[i]);
	}
	RF_Select_Not;
}

/* Configura la longitud de los bloques de datos de RX (0-32 Bytes)*/
void Set_RX_Length(uint8_t P1_LENGTH)
{
	RF_Select;
	SPI_Send(W_REGISTER|RX_PW_P1); 
	SPI_Send(P1_LENGTH); 
	RF_Select_Not;
	
	PP1_Length = P1_LENGTH;
}


/*	Configura los canales de RX habilitados. 
	(No modificar a menos que se desee tener ms canales de RX) */
Set_Enabled_RX_Addresses(uint8_t cfg_Rx)
{
	RF_Select;
	SPI_Send(W_REGISTER|EN_RXADDR); //Envia Comando de configuracion de RX addresses.
	SPI_Send(cfg_Rx); 
	RF_Select_Not;
}

/* Inicializa el nRF24L01+ con una velocidad de 2Mbps, y direcciones de 5 Bytes
	MODE		->	Configura el modo RX o TX. True para TX, False para RX.
	ADDR_P0		->	Direccin de TX (5-Bytes).
	ADDR_P1		->	Direccin de RX (5-Bytes).
	RF_CHANNEL	->	Canal de RF 2400MHZ - 2527 MHZ (introducir valor entre 0-127)
*/
void NRF24L01_Init(bool MODE, unsigned char ADDR_P0[5], 
unsigned char ADDR_P1[5], uint8_t RF_CHANNEL, uint8_t P0_LENGTH,
uint8_t P1_LENGTH, uint8_t RF_SPEED)
{	
	Set_RF_Channel(RF_CHANNEL);
	
	Set_TX_Address(ADDR_P0);
	
	Set_RX_Address(ADDR_P1);
	
	Set_Enabled_RX_Addresses(0x03);
	
 	Set_TX_Length(P0_LENGTH);
	
 	Set_RX_Length(P1_LENGTH);
	
	switch (RF_SPEED)
	{
		case 1:
		RF_SPEED = RF_SETUP_SPEED_2Mbps;
		break;
		
		case 2:
		RF_SPEED = RF_SETUP_SPEED_1Mbps;
		break;
		
		case 3:
		RF_SPEED = RF_SETUP_SPEED_250Kbps;
		break;
		
		default:
		RF_SPEED = RF_SETUP_SPEED_2Mbps;
	}
	
	
	Set_RF_Dataspeed(RF_SPEED);
	
	RF_Select;
	SPI_Send(W_REGISTER|SETUP_RETR); 
	SPI_Send(0x55); 
	RF_Select_Not;

	if (MODE==false)
	{
		Set_RX_Mode();
		
	}
	
	else if (MODE==true)
	{
		
		Set_TX_Mode();
		
	}
	

		Clean_Rx_Fifo();
		Clean_Tx_Fifo();
		_delay_ms(2);
}


/* Lee los datos recibidos va RF (bloques de datos estticos)
	readData[]	->	Arreglo donde se almacenar (por referencia)  el valor 
					ledo por RF. Su tamao debe ser igual o mayor al 
					definido en Set_RX_Length().
*/
void Read_RF_Data(unsigned char readData[])
{
	int8_t i;
	unsigned char *ptr= readData;
	
	if(sizeof(readData)<PP1_Length)
	{
		return;
	}
	
	RF_CE_Low;
	RF_Select;
	SPI_Send(R_RX_PAYLOAD);
	for(i=0;i<PP1_Length;i++)
	{
		ptr[i]=SPI_Send(NOP_RF);
	}
	RF_Select_Not;
}



/* Lee el valor de Bytes recibidos va RF (bloques de datos dinmicos) */
uint8_t Read_ACK_Data_Width()
{
	uint8_t width;
	
	RF_Select;
	SPI_Send(R_RX_PL_WID);
	width = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	return width;
}

/* Lee los datos recibidos va RF (bloques de datos dinmicos)

	RX_buffer[]	->	Arreglo donde se almacenar (por referencia)  el valor 
					ledo por RF. Su tamao debe ser igual o mayor al 
					recibido (se recomienda que su valor sea de 32 Bytes)
					
	Regresa el valor de los bytes recibidos por RF.
*/
uint8_t Read_Dynamic_RF_Data(unsigned char RX_buffer[])
{
	
	uint8_t i,long_datos_ACK;
	unsigned char *ptr= RX_buffer;
	
	long_datos_ACK = Read_ACK_Data_Width();
	
	
	if (long_datos_ACK > 32)
	{
		Clean_Rx_Fifo();
		return 0;
	}
	
	
	RF_Select;
	SPI_Send(R_RX_PAYLOAD);
	for(i=0;i<long_datos_ACK;i++)
	{
		ptr[i]=SPI_Send(0x00);
	}
	RF_Select_Not;
	


	return long_datos_ACK;

}

/*	Asigna un bloque de datos en el paquete de acuse de recibo (ACK)
	Se debe tener habilitado DPL con la funcin Enable_DPL() */
void Set_ACK_Data(uint8_t ACK_Data[])
{
	
	if (sizeof(ACK_Data)>32)
	{
		return;
	}
	
	uint8_t i;
	
	RF_Select;
	SPI_Send(W_ACK_PAYLOAD_PP0);
	for(i=0;i<sizeof(ACK_Data);i++)
	{
		SPI_Send(ACK_Data[i]);
	}
	RF_Select_Not;

}

/*	Habilita la funcin Dynamic Payload Length  y ACK Payload del nRF24L01+*/
bool Enable_DPL()
{
	
	unsigned char temp, buffer;
	
	RF_Select;
	SPI_Send(R_REGISTER|DYNPD);
	temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	buffer = temp | 0x3F;
	
	RF_Select;
	SPI_Send(W_REGISTER|DYNPD);
	SPI_Send(buffer);
	RF_Select_Not;
	
	RF_Select;
	SPI_Send(R_REGISTER|DYNPD);
	temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	if (temp == 0)
	{
		buffer = 0x73;
		
		RF_Select;
		SPI_Send(ACTIVATE);
		SPI_Send(buffer);
		RF_Select_Not;
		
		RF_Select;
		SPI_Send(R_REGISTER|DYNPD);
		temp = SPI_Send(NOP_RF);
		RF_Select_Not;
		
		buffer = temp | 0x3F;
		
		RF_Select;
		SPI_Send(W_REGISTER|DYNPD);
		SPI_Send(buffer);
		RF_Select_Not;
		
	}

	RF_Select;
	SPI_Send(R_REGISTER|FEATURE);
	temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	buffer = temp | 0x06;
	
	RF_Select;
	SPI_Send(W_REGISTER|FEATURE);
	SPI_Send(buffer);
	RF_Select_Not;
	
	return true;
}


/*	Deshabilita la funcin Dynamic Payload Length  y ACK Payload del nRF24L01+*/
bool Disable_DPL()
{
	
	RF_Select;
	SPI_Send(W_REGISTER|DYNPD);
	SPI_Send(0x00);
	RF_Select_Not;
	
	RF_Select;
	SPI_Send(W_REGISTER|FEATURE);
	SPI_Send(0x00);
	RF_Select_Not;
	
	return false;
}

/* Habilita las interrupciones por recepcin en IRQ. */
void Enable_RX_IRQ()
{
	unsigned char Config_Temp;
	
	RF_Select;
	SPI_Send(R_REGISTER|CONFIG_RF); 
	Config_Temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	Config_Temp &= 0x3F;
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(Config_Temp);
	RF_Select_Not;
}

/* Habilita las interrupciones por tranmsin y retransmisin en IRQ. */
void Enable_TX_IRQ()
{
	unsigned char Config_Temp;
	
	RF_Select;
	SPI_Send(R_REGISTER|CONFIG_RF); 
	Config_Temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	Config_Temp &= 0x4F;
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(Config_Temp);
	RF_Select_Not;
}

/* Habilita todas las interrupciones en IRQ. */
void Enable_All_IRQ()
{
	unsigned char Config_Temp;
	
	RF_Select;
	SPI_Send(R_REGISTER|CONFIG_RF); 
	Config_Temp = SPI_Send(NOP_RF);
	RF_Select_Not;
	
	Config_Temp &= 0x0F;
	
	RF_Select;
	SPI_Send(W_REGISTER|CONFIG_RF); 
	SPI_Send(Config_Temp);
	RF_Select_Not;
}


/*	Transmite un dato de longitud dinmica (1-32 bytes).
	Se debe activar el Dynamic Payload Length para poder utilizar sta funcin. 
	
	writeData[]	->		Arreglo que contiene  los datos a enviar por RF. 
						Su tamao no puede ser mayor a 32 Bytes.
	
	bytes		->		Nmero de Bytes a envar del arreglo writeData[].
						Si se desean enviar todos, se puede utilizar la
						funcin sizeof(writeData[]).
	
*/
void Transmit_Dynamic_RF_Data(uint8_t writeData[], uint8_t bytes) 
{
	uint8_t i;	

	if (sizeof(writeData)>32||bytes >32)
	{
		return;
	}

	RF_Select;
	SPI_Send(W_TX_PAYLOAD);
	for(i=0;i<bytes;i++)
	{
	SPI_Send(writeData[i]);
	}
	RF_Select_Not;
	
	RF_CE_High;
	_delay_us(30);
	RF_CE_Low;
}

/*	Transmite un dato de longitud fija (1-32 bytes).
	
	writeData[]	->		Arreglo que contiene  los datos a enviar por RF. 
						Su tamao debe ser igual al definido en Set_TX_Length().
	
*/
void Transmit_RF_Data(unsigned char writeData[])
{
	uint8_t i;

	if (sizeof(writeData)>32)
	{
		return;
	}

	RF_Select;
	SPI_Send(W_TX_PAYLOAD);
	for(i=0;i<PP0_Length;i++)
	{
		SPI_Send(writeData[i]);
	}
	RF_Select_Not;
	
	RF_CE_High;
	_delay_us(30);
	RF_CE_Low;
}
